Stan Math Library: stan/math/prim/scal/prob/cauchy_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

cauchy_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_CAUCHY_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_CAUCHY_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/log1p.hpp>
13 #include <stan/math/prim/scal/fun/square.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <boost/random/cauchy_distribution.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <limits>
19
20 namespace stan {
21
22 namespace math {
23
36 template <typename T_y, typename T_loc, typename T_scale>
37 typename return_type<T_y, T_loc, T_scale>::type
38 cauchy_cdf(const T_y& y, const T_loc& mu, const T_scale& sigma) {
39 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
40 T_partials_return;
41
42 // Size checks
43 if ( !( stan::length(y) && stan::length(mu)
44 && stan::length(sigma) ) )
45 return 1.0;
46
47 static const char* function("stan::math::cauchy_cdf");
48
49 using stan::math::check_positive_finite;
50 using stan::math::check_finite;
51 using stan::math::check_not_nan;
52 using stan::math::check_consistent_sizes;
53 using boost::math::tools::promote_args;
54 using stan::math::value_of;
55
56 T_partials_return P(1.0);
57
58 check_not_nan(function, "Random variable", y);
59 check_finite(function, "Location parameter", mu);
60 check_positive_finite(function, "Scale parameter", sigma);
61 check_consistent_sizes(function,
62 "Random variable", y,
63 "Location parameter", mu,
64 "Scale Parameter", sigma);
65
66 // Wrap arguments in vectors
67 VectorView<const T_y> y_vec(y);
68 VectorView<const T_loc> mu_vec(mu);
69 VectorView<const T_scale> sigma_vec(sigma);
70 size_t N = max_size(y, mu, sigma);
71
72 OperandsAndPartials<T_y, T_loc, T_scale>
73 operands_and_partials(y, mu, sigma);
74
75 // Explicit return for extreme values
76 // The gradients are technically ill-defined, but treated as zero
77 for (size_t i = 0; i < stan::length(y); i++) {
78 if (value_of(y_vec[i]) == -std::numeric_limits<double>::infinity())
79 return operands_and_partials.value(0.0);
80 }
81
82 // Compute CDF and its gradients
83 using std::atan;
84 using stan::math::pi;
85
86 // Compute vectorized CDF and gradient
87 for (size_t n = 0; n < N; n++) {
88 // Explicit results for extreme values
89 // The gradients are technically ill-defined, but treated as zero
90 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
91 continue;
92 }
93
94 // Pull out values
95 const T_partials_return y_dbl = value_of(y_vec[n]);
96 const T_partials_return mu_dbl = value_of(mu_vec[n]);
97 const T_partials_return sigma_inv_dbl = 1.0 / value_of(sigma_vec[n]);
98
99 const T_partials_return z = (y_dbl - mu_dbl) * sigma_inv_dbl;
100
101 // Compute
102 const T_partials_return Pn = atan(z) / pi() + 0.5;
103
104 P *= Pn;
105
106 if (!is_constant_struct<T_y>::value)
107 operands_and_partials.d_x1[n]
108 += sigma_inv_dbl / (pi() * (1.0 + z * z) * Pn);
109 if (!is_constant_struct<T_loc>::value)
110 operands_and_partials.d_x2[n]
111 += - sigma_inv_dbl / (pi() * (1.0 + z * z) * Pn);
112 if (!is_constant_struct<T_scale>::value)
113 operands_and_partials.d_x3[n]
114 += - z * sigma_inv_dbl / (pi() * (1.0 + z * z) * Pn);
115 }
116
117 if (!is_constant_struct<T_y>::value) {
118 for (size_t n = 0; n < stan::length(y); ++n)
119 operands_and_partials.d_x1[n] *= P;
120 }
121 if (!is_constant_struct<T_loc>::value) {
122 for (size_t n = 0; n < stan::length(mu); ++n)
123 operands_and_partials.d_x2[n] *= P;
124 }
125 if (!is_constant_struct<T_scale>::value) {
126 for (size_t n = 0; n < stan::length(sigma); ++n)
127 operands_and_partials.d_x3[n] *= P;
128 }
129
130 return operands_and_partials.value(P);
131 }
132 }
133 }
134 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::atan
fvar< T > atan(const fvar< T > &x)
Definition: atan.hpp:12
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::cauchy_cdf
return_type< T_y, T_loc, T_scale >::type cauchy_cdf(const T_y &y, const T_loc &mu, const T_scale &sigma)
Calculates the cauchy cumulative distribution function for the given variate, location, and scale.
Definition: cauchy_cdf.hpp:38
log1p.hpp
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.